unit PortIOVFD;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, Math, Portio, ExtCtrls;

type
  TPortIOVFD = class(TComponent)
  private
    { Private-Deklarationen }
    BA: word;          // Basisadresse
//    ActiveScreen: byte;   // aktiver Bildschirm
    DC: array[0..9, 0..191] of Byte;  //enthlt den Charsatz im Displayformat
    LightLevel: Byte;       // fr Fadeout/in
    FOut: boolean;          // FadeOut/In

    //fr Animationen
    framewidth: byte;  //Framebreite in Pixeln
    frame: word;   //aktuelles Frame
    frames: word;  // Anzahl Frames
    Bmap: TFileName; // Bitmapdateiname
    XPosit: word; //XPositionim Display

    //fr Uhren
    Digital: boolean;
    XUhr: byte;  // bei Analoguhren (64x64 Pixel) Position auf dem Display


    //Objekte
    Bmp: TBitmap;              // Bitmapobjekt fr Animation
    FadeTimer: TTimer;         //MatrixTimerObjekt
    AnimateTimer: TTimer;      //AnimationsTimerObjekt
    ClockBitmap: TBitmap;      //UhrBitmap
    ClockTimer: TTimer;        //UhrTimer
  protected
    { Protected-Deklarationen }
    procedure outport(Adresse: word; wert: byte);          { Portausgabe }
    procedure DoppelPunkt;       { Zeichent Doppelpunkte fr die Digitaluhr }
    procedure LoadChars;         {Ldt Zeichensatz fr digitale Uhranzeige}
    procedure AnimateTimerEvent(Sender: TObject);
    procedure AnimateBitmap(bmp: TBitmap;xPos,Frame, FrameWidth: word; Farbe: TColor);       { }
    procedure FadeTimerEvent(Sender: TObject);
    procedure ClockTimerEvent(Sender: TObject);
    procedure RefreshTime;  { Aktualisiert die Uhrzeitausgabe }
    procedure BitmapChangeEvent(Sender: TObject);  // RefreshProzedurfr BitmapObjekt->Display
  public
    { Public-Deklarationen }
    PortIOComponent: TDLPortIO;
    Bitmap: TBitmap;

    constructor Create (AOwner: TComponent); override;
    destructor Destroy; override;
    procedure InitVFD;                                { Display initialisieren }

    procedure WriteCommand(c: byte);
    procedure WriteData(d: byte);

    procedure SetCursor(C_low,C_high: byte);       { Cursor setzen }
    procedure ClearScreen(ScreenNr: byte);      { Screen mit "ScreenNr" lschen }
    procedure SelectScreen(Screen: byte);       { Aktiven Bildschirm whlen (3=beide) }


    procedure PaintString(s: string; col,row: byte);       { String an Pos col,row auf Char-Display ausgeben }
    procedure PaintBitmapRect(Source: TBitmap;xDisplay,yDisplay,width,xOffset: byte; Farbe: TColor);       { Zeichenflche auf Display an Pos x,y mit Breite width ausgeben }
    procedure ClearBitmap;       { Bitmap lschen }
    procedure PaintCanvasRect(Source: TCanvas;xDisplay,yDisplay,width,xOffset: byte; Farbe: TColor);       { Zeichenflche auf Display an Pos x,y mit Breite width ausgeben }
    procedure SetPixelbyte(PixelByte,x,y: byte);       { Pixelbyte auf Display an x,y malen }

    procedure Animate(FileName: TFilename; XPosition, AnimationSpeed: word);
    procedure PauseAnimation;
    procedure StopAnimation;
    procedure Fade(off: boolean; Speed: word);

    procedure Time(analog: boolean; Xposition: byte);
    procedure stopClock;


  published
    { Published-Deklarationen }
    property LPT: word read BA write BA;

end;

const
{ Konstanten fr Zugriff auf Steuerleitungen }
     nWR = 1;       { Wertigkeit /WR @ STROBE   }
     nRD = 2;       { Wertigkeit /RD @ AUTOFEED }
     CnD = 8;       { Wertigkeit C/D @ SELECT   }
     Maske = 11;    { XOR-Bitmaske fr Ausgabe auf Steuerleitungen }

{ Displaydimensionen }
     TEXT_SCREEN_WIDTH = 128;
     TEXT_SCREEN_HEIGHT = 8;
     GRAPHICS_SCREEN_WIDTH = 256;
     GRAPHICS_SCREEN_HEIGHT = 64;

{ Kommando Definitionen, Werte in Hexadezimaldarstellung}
     CMD_SCR_OFF =     $00;    { Beide Screens aus }
     CMD_SCR1_ON =     $01;    { Screen 1 an }
     CMD_SCR2_ON =     $02;    { Screen 2 an }
     CMD_SCR1and2_ON = $03;    { Beide Screens an }

     CMD_CUR_INC =     $04;    { Cursor fhrt automatisch weiter *DEFAULT }
     CMD_CUR_HOLD =    $05;    { Cursor stoppt }

     CMD_SCR2_CHAR =   $06;    { Screen 2 als Textdisplay }
     CMD_SCR2_GFX =    $07;    { Screen 2 als Grafikdisplay }

     CMD_WRITE_DATA =  $08;    { Schreibkommando }
     CMD_READ_DATA =   $09;    { Lesekommando }

     CMD_SCR1_lowADR =  $0A;   { Kommando: untere Adresse Screen 1 }
     CMD_SCR1_highADR = $0B;   { Kommando: obere Adresse Screen 1  }
     CMD_SCR2_lowADR =  $0C;   { Kommando: untere Adresse Screen 2 }
     CMD_SCR2_highADR = $0D;   { Kommando: obere Adresse Screen 2  }

     CMD_CUR_lowADR =   $0E;   { Kommando: untere Adresse Cursor }
     CMD_CUR_highADR =  $0F;   { Kommando: obere Adresse Cursor  }

     CMD_SCR_OR =       $10;   { ODER-Verknpfung }
     CMD_SCR_XOR =      $11;   { NICHT-ODER-Verknpfung }
     CMD_SCR_AND =      $12;   { UND-Verknpfung }

     CMD_LIGHT_Lev1 =   $18;   { volle Helligkeit }
     CMD_LIGHT_Lev2 =   $19;   { 87,5% Helligkeit }
     CMD_LIGHT_Lev3 =   $1A;   { 75%   Helligkeit }
     CMD_LIGHT_Lev4 =   $1B;   { 62,5% Helligkeit }
     CMD_LIGHT_Lev5 =   $1C;   { inoffiziell }
     CMD_LIGHT_Lev6 =   $1D;   { inoffiziell }
     CMD_LIGHT_Lev7 =   $1E;   { inoffiziell }
     CMD_LIGHT_Lev8 =   $1F;   { inoffiziell }

{ Adressbereiche SCR1 = $0000, SCR2 = $0800 }
     SCR1_L = $00;      { untere Startaddresse Screen 1 }
     SCR1_H = $00;      { obere Startaddresse Screen 1  }
     SCR2_L = $00;      { untere Startaddresse Screen 2 }
     SCR2_H = $08;      { obere Startaddresse Screen 2  }




procedure Register;

implementation

procedure TPortIOVFD.Fade(off: boolean; Speed: word);
begin
      selectscreen(3); // beide
      FadeTimer.Enabled:=false;
      FadeTimer.Interval:=Speed;
      FadeTimer.OnTimer:=FadeTimerEvent;
      Fout:= off;
      FadeTimer.Enabled:=true;
end;

procedure TPortIOVFD.FadeTimerEvent(Sender: TObject);
begin
     If Fout= true then LightLevel:=LightLevel+1
     else LightLevel:=LightLevel-1;

     if (LightLevel> CMD_LIGHT_Lev8) then begin
      LightLevel:=CMD_Light_Lev8;
      FadeTimer.enabled:=false;
     end;
     if (LightLevel< CMD_LIGHT_Lev1) then begin
      LightLevel:=CMD_Light_Lev1;
      FadeTimer.enabled:=false;
     end;
     WriteCommand(LightLevel);
end;

procedure TPortIOVFD.ClockTimerEvent(Sender: TObject);
const
     strichle = 7;   // Lnge der Stundenstriche am Uhrrand
     SekRad= 31;     // Lnge des Sekundenzeigers
     MinRad= 20;     // Lnge des Minutenzeigers
     HrRad = 15;     // Lnge des Stundenzeigers
var
   h,m,s,ms: word;
begin
     if digital = false then begin        // zeigt Analoguhr an
     with ClockBitmap.Canvas do begin     // zeichnen auf Uhrenbitmap
     Pen.Width:=2;
     //Kreis
     Ellipse(0,0,64,64);
     //Strichle
     Pen.Width:=1;
     MoveTo(32,0);
     LineTo(32,strichle);
     MoveTo(32,63);
     LineTo(32,63-strichle);
     MoveTo(0,32);
     LineTo(strichle,32);
     MoveTo(63,32);
     LineTo(63-strichle,32);
     MoveTo(4,17);
     LineTo(12,21);
     MoveTo(17,4);
     LineTo(21,12);
     MoveTo(59,17);
     LineTo(51,21);
     MoveTo(46,4);
     LineTo(42,12);
     MoveTo(4,63-17);
     LineTo(12,63-21);
     MoveTo(17,63-4);
     LineTo(21,63-12);
     MoveTo(59,63-17);
     LineTo(51,63-21);
     MoveTo(46,63-4);
     LineTo(42,63-12);
     //Zeit dekodieren
     decodetime(now,h,m,s,ms);
     if h>12 then h:=h-12;  //von 24h nach 12h - Format
     //Umrechnungen in Grad
     h:=h*6*5;
     m:=m*6;
     s:=s*6;

     Pen.Width:=1;
     moveto(32,32);
     lineto(32+ round(31*sin(DegToRad(s))) , 32-round(31*cos(DegToRad(s))) );

     Pen.Width:=1;
     moveto(32,32);
     lineto(32+ round(SekRad*sin(DegToRad(s))) , 32-round(SekRad*cos(DegToRad(s))) );

     Pen.Width:=2;
     moveto(32,32);
     lineto(32+ round(MinRad*sin(DegToRad(m))) , 32-round(MinRad*cos(DegToRad(m))) );

     Pen.Width:=3;
     moveto(32,32);
     lineto(32+ round(HrRad*sin(DegToRad(h+m/12)) ) , 32-round(HrRad*cos(DegToRad(h+m/12))) );
     end;

     SelectScreen(3);
     PaintBitmapRect(ClockBitmap,xUhr,0,64,0,clblack);
     end // digital = false
     else RefreshTime;      // zeigt Uhrzeit digital an

end;


procedure TPortIOVFD.Animate(FileName: TFilename; XPosition, AnimationSpeed: word);
begin

   //Dateizuweisung
   Bmap:=FileName;    //Bmap = globale Variable

   //ermittle Anzahl der Frames aus Dateiname
   frames:=strtoint(copy(extractfilename(bmap),0,2));

   bmp.LoadFromFile(Bmap); //Datei laden

   framewidth:=bmp.width div frames;  //Framebreite berechnen

   selectscreen(3);   // beide Screens an
   XPosit:=XPosition;
   AnimateTimer.Enabled:=false;
   AnimateTimer.Interval:=AnimationSpeed;
   AnimateTimer.Enabled:=true;

end;

procedure TPortIOVFD.PauseAnimation;
begin
     if bmp<>nil then AnimateTimer.enabled:=not AnimateTimer.enabled;    //nil in bmp wrde bei bei Animation zu Exception fhren
end;

procedure TPortIOVFD.StopAnimation;
begin
     AnimateTimer.enabled:=false;
     frame:=0;
end;

procedure TPortIOVFD.AnimateTimerEvent(Sender: TObject);
begin
     AnimateBitmap(bmp,XPosit,Frame,Framewidth, clBlack);
     inc(frame);
     if frame >= frames then frame:=0;
end;


procedure TPortIOVFD.RefreshTime;
var
   xoff,yoff: Word;  //wo?
   x,y: Word; //Koordinaten
   i: word;    //Zhler
   ziffer: byte; //Ziffer
   h,m,s,ms: word;
begin
     x:=0;
     y:=0;

     xoff:=220;
     yoff:=2;

     decodetime(now,h,m,s,ms);

     //Sekunden-Einer
     ziffer:=s mod 10;
     for i:=0 to 191 do begin
      setpixelbyte(DC[ziffer,i],xoff+x,yoff+y);
      y:=y+1;
      if y=6 then begin
       y:=0;
       x:=x+1;
      end;
     end;

      if Ziffer = 0 then begin
       xoff:=xoff-70;
       //Sekunden-Zehner
       ziffer:=s div 10;
       for i:=0 to 191 do begin
        setpixelbyte(DC[ziffer,i],xoff+x,yoff+y);
        y:=y+1;
        if y=6 then begin
         y:=0;
         x:=x+1;
        end;
       end;
      end; //Ziffer =0

      if Ziffer = 0 then begin
       xoff:=xoff-80;
       //Minuten-Einer
       ziffer:=m mod 10;
       for i:=0 to 191 do begin
        setpixelbyte(DC[ziffer,i],xoff+x,yoff+y);
        y:=y+1;
        if y=6 then begin
         y:=0;
         x:=x+1;
        end;
       end;
      end; //Ziffer =0

      if Ziffer = 0 then begin
       xoff:=xoff-70;
       //Minuten-Zehner
       ziffer:=m div 10;
       for i:=0 to 191 do begin
        setpixelbyte(DC[ziffer,i],xoff+x,yoff+y);
        y:=y+1;
        if y=6 then begin
         y:=0;
         x:=x+1;
        end;
       end;
      end; //Ziffer =0

      if Ziffer = 0 then begin
       xoff:=xoff-80;
       //Stunden-Einer
       ziffer:=h mod 10;
       for i:=0 to 191 do begin
        setpixelbyte(DC[ziffer,i],xoff+x,yoff+y);
        y:=y+1;
        if y=6 then begin
         y:=0;
         x:=x+1;
        end;
       end;
      end; //Ziffer =0

      if Ziffer = 0 then begin
       xoff:=xoff-70;
       //Stunden-Zehner
       ziffer:=h div 10;
       for i:=0 to 191 do begin
        setpixelbyte(DC[ziffer,i],xoff+x,yoff+y);
        y:=y+1;
        if y=6 then begin
         y:=0;
         x:=x+1;
        end;
       end;
      end; //Ziffer =0
end;

procedure TPortIOVFD.LoadChars;
var F: File of Byte;
    i: Word;   // Byteindex
    Fi: byte;  // Fileindex
    b: byte;   // (Pixel-)Byte
begin

  for Fi:=0 to 9 do begin

   AssignFile(F, extractfilepath(application.exename)+ inttostr(Fi)+'.dat');
   Reset(F);
   i:=0;
   while i<192 do begin
    Read(F, b);
    DC[Fi, i]:=b;
    i:=i+1;
   end;
   CloseFile(F);

  end; //Fi
end;

procedure TPortIOVFD.DoppelPunkt;       { Zeichent Doppelpunkte fr die Uhr }
begin
     SetPixelbyte(255,175,4);
     SetPixelbyte(255,174,4);
     SetPixelbyte(255,173,4);
     SetPixelbyte(255,172,4);
     SetPixelbyte(255,171,4);
     SetPixelbyte(255,170,4);

     SetPixelbyte(255,175,6);
     SetPixelbyte(255,174,6);
     SetPixelbyte(255,173,6);
     SetPixelbyte(255,172,6);
     SetPixelbyte(255,171,6);
     SetPixelbyte(255,170,6);


     SetPixelbyte(255,90,4);
     SetPixelbyte(255,89,4);
     SetPixelbyte(255,88,4);
     SetPixelbyte(255,87,4);
     SetPixelbyte(255,86,4);
     SetPixelbyte(255,85,4);

     SetPixelbyte(255,90,6);
     SetPixelbyte(255,89,6);
     SetPixelbyte(255,88,6);
     SetPixelbyte(255,87,6);
     SetPixelbyte(255,86,6);
     SetPixelbyte(255,85,6);

end;

procedure TPortIOVFD.Time(analog: boolean; Xposition: byte);
var
   xoff,yoff: Integer;  //wo?
   x,y: Word; //Koordinaten
   i: word;    //Zhler
   ziffer: byte; //Ziffer
   h,m,s,ms: word;
begin
     if analog = true then begin
      digital:=false;
      xUhr:=xPosition;
     end
     else begin  // digital
      clearscreen(1);    // GFX_Screen lschen
      digital:=true;
      DoppelPunkt;
     x:=0;
     y:=0;

     xoff:=220;
     yoff:=2;

     decodetime(now,h,m,s,ms);
     //ganze Uhrzeit zeichnen (alle Ziffern)
     //Sekunden-Einer
     ziffer:=s mod 10;
     for i:=0 to 191 do begin
      setpixelbyte(DC[ziffer,i],xoff+x,yoff+y);
      y:=y+1;
      if y=6 then begin
       y:=0;
       x:=x+1;
      end;
     end;

       xoff:=xoff-70;
       //Sekunden-Zehner
       ziffer:=s div 10;
       for i:=0 to 191 do begin
        setpixelbyte(DC[ziffer,i],xoff+x,yoff+y);
        y:=y+1;
        if y=6 then begin
         y:=0;
         x:=x+1;
        end;
       end;

       xoff:=xoff-80;
       //Minuten-Einer
       ziffer:=m mod 10;
       for i:=0 to 191 do begin
        setpixelbyte(DC[ziffer,i],xoff+x,yoff+y);
        y:=y+1;
        if y=6 then begin
         y:=0;
         x:=x+1;
        end;
       end;

       xoff:=xoff-70;
       //Minuten-Zehner
       ziffer:=m div 10;
       for i:=0 to 191 do begin
        setpixelbyte(DC[ziffer,i],xoff+x,yoff+y);
        y:=y+1;
        if y=6 then begin
         y:=0;
         x:=x+1;
        end;
       end;

       xoff:=xoff-80;
       //Stunden-Einer
       ziffer:=h mod 10;
       for i:=0 to 191 do begin
        setpixelbyte(DC[ziffer,i],xoff+x,yoff+y);
        y:=y+1;
        if y=6 then begin
         y:=0;
         x:=x+1;
        end;
       end;

       xoff:=xoff-70;
       //Stunden-Zehner
       ziffer:=h div 10;
       for i:=0 to 191 do begin
        setpixelbyte(DC[ziffer,i],xoff+x,yoff+y);
        y:=y+1;
        if y=6 then begin
         y:=0;
         x:=x+1;
        end;
       end;

     end;   // end else
     clocktimer.Enabled:= true;    // Uhrentimer aktivieren - dessen OnTimerEvent kmmert sich um Aktualisierung
end;

procedure TPortIOVFD.stopClock;
begin
     clocktimer.Enabled:= false;
end;

procedure TPortIOVFD.outport(Adresse: word; wert: byte);    { Portausgabe }
var
   DataWrite : Longword;  //Daten
begin

   DataWrite:=wert;

   try
      DataWrite:=Longword(DataWrite);  //Umwandlung in Longword (falls ntig)
   except
   end;

   try
    PortIOComponent.Port[Adresse]:=Byte(DataWrite and $FF);
   except
   end;
end;


procedure TPortIOVFD.WriteCommand(c: byte);
begin
     // setze C/D auf 1 fr "C"
     outport(BA+2,(CnD+nRD) XOR Maske);
     outport(BA+2,(CnD+nRD) XOR Maske);
     // lege Kommandobyte an Datenleitungen
     outport(Ba,c);
     outport(Ba,c);
     // setze /WR auf 1
     outport(BA+2,(CnD+nWR+nRD) XOR Maske);
     outport(BA+2,(CnD+nWR+nRD) XOR Maske);
     // /WR wieder rcksetzen
     outport(BA+2,(CnD+nRD) XOR Maske);
     outport(BA+2,(CnD+nRD) XOR Maske);
     // setze C/D wieder auf "D"
     outport(BA+2,(nRD+nWR) XOR Maske);
     outport(BA+2,(nRD+nWR) XOR Maske);
end;


procedure TPortIOVFD.WriteData(d: byte);
begin
     // setze C/D aud "D"
     outport(BA+2,(nRD+nWR) XOR Maske);
     outport(BA+2,(nRD+nWR) XOR Maske);
     // lege Datenbyte an Datenleitungen an
     outport(BA,d);
     outport(BA,d);
     // setze /WR
     outport(BA+2,(nWR+nRD) XOR Maske);
     outport(BA+2,(nWR+nRD) XOR Maske);
      // /WR wieder zurcksetzen
     outport(BA+2,(nRD) XOR Maske);
     outport(BA+2,(nRD) XOR Maske);
    // setze C/D wieder auf "C"
     outport(BA+2,(CnD+nRD+nWR) XOR Maske);
     outport(BA+2,(CnD+nRD+nWR) XOR Maske);
end;





procedure TPortIOVFD.SetCursor(C_low,C_high: byte);       { Cursor setzen }
begin
   WriteCommand(CMD_CUR_lowADR);
   WriteData(C_low);
   WriteCommand(CMD_CUR_highADR);
   WriteData(C_high);
end;

procedure TPortIOVFD.ClearBitmap;       { Bitmap lschen }
begin
     Bitmap.OnChange:=nil; //verhindern, da das Bitmap neu auf das Display gezeichnet wird
      Bitmap.Width:=0;     //Gre kurz auf Null setzen lscht den Inhalt schneller als FloodFill 
      Bitmap.Height:=0;
      Bitmap.Width:=256;
      Bitmap.Height:=64;
     Bitmap.OnChange:=BitmapChangeEvent;  //ab jetzt darf bei Bitmapnderungen wieder Ausgabe aud Display erfolgen
end;

procedure TPortIOVFD.ClearScreen(ScreenNr: byte);       { Screen mit "ScreenNr" lschen }
var
   i: integer;
begin
   // Berechne Startadresse
   if ScreenNr=1 then begin
    SetCursor(SCR1_L,SCR1_H);
   end
   else begin
    SetCursor(SCR2_L,SCR2_H);
   end;

   // setze Cursor in autoinc-mode
   WriteCommand(CMD_CUR_INC);

   // Schreibkommando
   WriteCommand(CMD_WRITE_DATA);

   //lschen
   if ScreenNr=1 then begin
     for i:=1 to (GRAPHICS_SCREEN_WIDTH*GRAPHICS_SCREEN_HEIGHT) do
      begin
       WriteData(0);
      end;
    end
    else begin
     for i:=1 to (TEXT_SCREEN_WIDTH*TEXT_SCREEN_HEIGHT) do
      begin
       WriteData(0);
      end;
   end;
   // setze Cursor in autohold-mode
   WriteCommand(CMD_CUR_HOLD);
end;


procedure TPortIOVFD.PaintString(s: string; col,row: byte);       { String an Pos col,row auf Char-Display ausgeben }
var
   a,b: byte;                    // bilden Adresse fr Cursorposition (a=CUR_low, b=CUR_high)
   position: word;               // Adresse als Word
   c: char;                      // Char
   i: integer;                   // for-Variable
begin
     // berprfen ob Parameter sinnvoll sind
     if col>(TEXT_SCREEN_WIDTH-1) then col:=(TEXT_SCREEN_WIDTH-1);
     if row>(TEXT_SCREEN_HEIGHT-1) then row:=(TEXT_SCREEN_HEIGHT-1);

     // Adresse berechnen
     position:=SCR2_L+(SCR2_H*$100);                    // Adresse des SCR2 als Offset...
     position:=position+ (row*TEXT_SCREEN_WIDTH)+col;   // ... + Position auf dem Display

     // ermittle CUR_low und CUR_high aus position
     a:=trunc(position mod $100);
     b:=trunc(position / $100);

     // setze Cursor in autoinc-mode
     WriteCommand(CMD_CUR_INC);

     // setze Cursor auf position
     SetCursor(a,b);

     // Schreibkommando
     WriteCommand(CMD_WRITE_DATA);

     // For-Schleife fr Zeichenweise Ausgabe des Strings
     for i:= 1 to length(s) do begin
      // kopiere Char aus String s
      c:=s[i];
      // c auf Display ausgeben
      WriteData(ord(c));
     end;
     // setze Cursor in autohold-mode
     WriteCommand(CMD_CUR_HOLD);
end;


procedure TPortIOVFD.SetPixelbyte(PixelByte,x,y: byte);       { Pixelbyte auf Display an x,y malen }
var
   a,b: byte;                    // bilden Adresse fr Cursorposition (a=CUR_low, b=CUR_high)
   position: word;               // Adresse als Word
begin
     // berprfen ob Parameter sinnvoll sind
     if x>(GRAPHICS_SCREEN_WIDTH-1) then x:=(GRAPHICS_SCREEN_WIDTH-1);
     if y>(GRAPHICS_SCREEN_HEIGHT-1) then y:=(GRAPHICS_SCREEN_HEIGHT-1);

     // Adresse berechnen
     position:=SCR1_L+(SCR1_H*$100);                    // Adresse des SCR1 als Offset...
     position:=position+ (x*8)+y;                       // ... + Position auf dem Display

     // ermittle CUR_low und CUR_high aus position
     a:=trunc(position mod $100);
     b:=trunc(position / $100);

     // setze Cursor auf position
     SetCursor(a,b);

     // Schreibkommando
     WriteCommand(CMD_WRITE_DATA);

     // sende Pixelbyte
     WriteData(PixelByte);

     // setze Cursor in autohold-mode
     WriteCommand(CMD_CUR_HOLD);
end;

procedure TPortIOVFD.SelectScreen(Screen: byte);       { Aktiven Bildschirm whlen (3=beide) }
begin
   if (Screen<=3)and (Screen>=0) then WriteCommand(Screen);
//   ActiveScreen:=screen;
end;


procedure TPortIOVFD.AnimateBitmap(bmp: TBitmap;xPos,Frame ,FrameWidth: word; Farbe: TColor);       { }
var
   x,y,i: word;                    // Zhlervariablen
   pixbyte: byte;                  // Pixelbyte
begin

     // Schreibkommando
     WriteCommand(CMD_WRITE_DATA);

     // Ausgabe
     for x:=0 to Framewidth do begin
     for y:=0 to 7 do begin

       // reset pixbyte
       pixbyte:=0;
       //pixbyte erstellen
       for i:=0 to 7 do begin
        if bmp.canvas.Pixels[x+Frame*Framewidth,(y*8)+7-i]=Farbe then pixbyte:=pixbyte+ trunc(Power(2,i));
       end;
       SetPixelByte(pixbyte,x+xpos,y);

     end;
     end;

     // setze Cursor in autohold-mode
     WriteCommand(CMD_CUR_HOLD);

end;

procedure TPortIOVFD.BitmapChangeEvent(Sender: TObject);  // RefreshProzedurfr BitmapObjekt->Display
var
   x,y,i: word;                    // Zhlervariablen
   pixbyte: byte;                  // Pixelbyte
begin

     // Schreibkommando
     WriteCommand(CMD_WRITE_DATA);

     // Ausgabe
     for x:=0 to 255 do begin
     for y:=0 to 7 do begin
       // reset pixbyte
       pixbyte:=0;
       //pixbyte erstellen
       for i:=0 to 7 do begin
        if bitmap.canvas.Pixels[x,(y*8)+7-i]=clBlack then pixbyte:=pixbyte+ trunc(Power(2,i));
       end;
       SetPixelByte(pixbyte,x,y);
     end;
     end;

     // setze Cursor in autohold-mode
     WriteCommand(CMD_CUR_HOLD);
end;

procedure TPortIOVFD.PaintBitmapRect(Source: TBitmap;xDisplay,yDisplay,width,xOffset: byte; Farbe: TColor);       { Zeichenflche auf Display an Pos x,y mit Breite width ausgeben }
var
   x,y,i: word;                    // Zhlervariablen
   pixbyte: byte;                  // Pixelbyte
begin

     // Schreibkommando
     WriteCommand(CMD_WRITE_DATA);

     // Ausgabe
     for x:=0 to width do begin
     for y:=0 to 7-yDisplay do begin

       // reset pixbyte
       pixbyte:=0;
       //pixbyte erstellen
       for i:=0 to 7 do begin
        if Source.canvas.Pixels[x+xOffset,(y*8)+7-i]=Farbe then pixbyte:=pixbyte+ trunc(Power(2,i));
       end;
       SetPixelByte(pixbyte,x+xDisplay,y+yDisplay);

     end;
     end;

     // setze Cursor in autohold-mode
     WriteCommand(CMD_CUR_HOLD);

end;


procedure TPortIOVFD.PaintCanvasRect(Source: TCanvas;xDisplay,yDisplay,width,xOffset: byte; Farbe: TColor);       { Zeichenflche auf Display an Pos x,y mit Breite width ausgeben }
var
   x,y,i: word;                    // Zhlervariablen
   pixbyte: byte;                  // Pixelbyte
begin

     // Schreibkommando
     WriteCommand(CMD_WRITE_DATA);

     // Ausgabe
     for x:=0 to width do begin
     for y:=0 to 7-yDisplay do begin

       // reset pixbyte
       pixbyte:=0;
       //pixbyte erstellen
       for i:=0 to 7 do begin
        if Source.Pixels[x+xOffset,(y*8)+7-i]=Farbe then pixbyte:=pixbyte+ trunc(Power(2,i));
       end;
       SetPixelByte(pixbyte,x+xDisplay,y+yDisplay);

     end;
     end;

     // setze Cursor in autohold-mode
     WriteCommand(CMD_CUR_HOLD);

end;




procedure TPortIOVFD.InitVFD;       { Display initialisieren }
begin
     // setze Screen1-Startadresse
     WriteCommand(CMD_SCR1_lowADR);
     WriteData(SCR1_L);
     WriteCommand(CMD_SCR1_highADR);
     WriteData(SCR1_H);
     // setze Screen2-Startadresse
     WriteCommand(CMD_SCR2_lowADR);
     WriteData(SCR2_L);
     WriteCommand(CMD_SCR2_highADR);
     WriteData(SCR2_H);
     // setze Cursor in autohold-mode
     WriteCommand(CMD_CUR_HOLD);
     // Volle Helligkeit
     LightLevel:=CMD_LIGHT_Lev1;
     WriteCommand(CMD_LIGHT_Lev1);
     // Charsatz laden
     LoadChars;


end;





constructor TPortIOVFD.Create (AOwner: TComponent);
begin
     inherited Create (AOwner);
      BA:=$378;                 // Default-Wert beim Laden
     // ActiveScreen:=0;

     try
      //Animationstimer erstellen
      AnimateTimer:=TTimer.create(self);
      AnimateTimer.enabled:=false;
      AnimateTimer.OnTimer:=AnimateTimerEvent;
     except
      AnimateTimer.free;
     end;

     try
      //AnimationsBitmapObjekt erstellen
      bmp:=TBitmap.create;
     except
      bmp.free;
     end;

     try
      //FadeIn/Out Timer
      FadeTimer:=TTimer.Create(self);
      FadeTimer.Enabled:=false;
     except
      FadeTimer.free;
     end;

     try
      //BitmapObjekt erstellen
      Bitmap:=TBitmap.Create;
      Bitmap.Width:=256;
      Bitmap.Height:=64;
      Bitmap.OnChange:=BitmapChangeEvent;
     except
      Bitmap.free;
     end;

     try
      //ClockBitmapObjekt erstellen
      ClockBitmap:=TBitmap.Create;
      ClockBitmap.width:=64;
      ClockBitmap.height:=64;
     except
      ClockBitmap.Free;
     end;

     try
      //UhrTimer erstellen
      ClockTimer:= TTimer.Create(self);
      ClockTimer.enabled:=false;
      ClockTimer.OnTimer:=ClockTimerEvent;
     except
      ClockTimer.free;
     end;

     try
      //AnimationsBitmapObjekt erstellen
      Bmp:=TBitmap.Create;
     except
      Bmp.free;
     end;
end;

destructor TPortIOVFD.Destroy;
begin
     if AnimateTimer <> nil then AnimateTimer.Free;
     if bmp <> nil then bmp.free;
     if FadeTimer <> nil then FadeTimer.free;
     if bitmap <> nil then Bitmap.free;
     if ClockBitmap <> nil then ClockBitmap.Free;
     if ClockTimer <> nil then clocktimer.Free;

     inherited Destroy;
end;

procedure Register;
begin
  RegisterComponents('Hardware', [TPortIOVFD]);
end;

end.
