unit setupUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, IniFiles,
  StdCtrls, ComCtrls, Buttons;

type
  TSetupForm = class(TForm)
    GroupBox1: TGroupBox;
    PlayOnlyOnIdleBox: TCheckBox;
    IdleTimeBar: TScrollBar;
    IdleLevelBar: TTrackBar;
    Label2: TLabel;
    Label3: TLabel;
    IdleTimeLabel: TLabel;
    IdleLevelLabel: TLabel;
    GroupBox2: TGroupBox;
    ShowTimeBar: TScrollBar;
    Label4: TLabel;
    ShowTimeLabel: TLabel;
    OverviewTimeBar: TScrollBar;
    Label5: TLabel;
    OverviewTimeLabel: TLabel;
    GroupBox3: TGroupBox;
    Label1: TLabel;
    PortEdit: TEdit;
    OKButton: TBitBtn;
    AbortButton: TBitBtn;
    Label6: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure IdleLevelBarChange(Sender: TObject);
    procedure IdleTimeBarChange(Sender: TObject);
    procedure ShowTimeBarChange(Sender: TObject);
    procedure OverviewTimeBarChange(Sender: TObject);
    procedure AbortButtonClick(Sender: TObject);
    procedure OKButtonClick(Sender: TObject);
  private
    ShowTime: Integer;
    OverviewTime: Integer;
    LPTPort: Word;
    ToDoList: string;
    PlayOnlyOnIdle: Boolean;
    IdleTime: Integer;
    IdleLevel: Integer;
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
  end;

var
  SetupForm: TSetupForm;

implementation

{$R *.DFM}

procedure TSetupForm.FormCreate(Sender: TObject);
var
  VFDIni: TIniFile;
  IniName: string;
begin
    try
    IniName:=extractfilepath(application.ExeName)+'Vfd.ini';

    VFDIni := TIniFile.Create(IniName);
    with VFDIni do begin
       SetupForm.ShowTime:=ReadInteger('EXTRAS', 'ShowTime', 3000);
       SetupForm.OverviewTime:=ReadInteger('EXTRAS', 'OverviewTime', 6000);
       SetupForm.LPTPort:=ReadInteger('SETTINGS', 'Port', 888);
       SetupForm.ToDoList:=ReadString('LIST', 'Listname', '');
       SetupForm.PlayOnlyOnIdle:=ReadBool('ANIMATIONS', 'PlayOnlyOnIdle', false);
       SetupForm.IdleTime:=ReadInteger('ANIMATIONS', 'IdleTime', 45);
       SetupForm.IdleLevel:=ReadInteger('ANIMATIONS', 'IdleLevel', 25);
    end
    finally
     VFDIni.Free;
    end;

    //Port
    PortEdit.Text:=inttohex(LPTPort,3);

    //IdleLevel
    IdleLevelBar.Position:=100-IdleLevel;
    IdleLevelLabel.caption:=inttostr(IdleLevel)+'%';
    //IdleTime
    IdleTimeBar.Position:=IdleTime;
    IdleTimeLabel.caption:=copy(timetostr(encodetime(0,IdleTime*2 div 60,IdleTime*2 mod 60,0)),4,5)+' Min.';
    //PlayOnlyOnIdle
    PlayOnlyOnIdleBox.Checked:=PlayOnlyOnIdle;

    //ShowTime
    ShowTimeLabel.caption:=copy(timetostr(encodetime(0,(ShowTime div 1000) div 60,(ShowTime div 1000) mod 60,0)),4,5)+' Min.';
    ShowTimeBar.Position:=ShowTime div 1000;

    //OverviewTime
    OverviewTimeLabel.caption:=copy(timetostr(encodetime(0,(OverviewTime div 1000) div 60,(OverviewTime div 1000) mod 60,0)),4,5)+' Min.';
    OverviewTimeBar.Position:=OverviewTime div 1000;

end;

procedure TSetupForm.IdleLevelBarChange(Sender: TObject);
begin
    IdleLevelLabel.caption:=inttostr(100-IdleLevelBar.Position)+'%';
    IdleLevel:=100-IdleLevelBar.Position;
end;

procedure TSetupForm.IdleTimeBarChange(Sender: TObject);
begin
    IdleTime:=IdleTimeBar.position;
    IdleTimeLabel.caption:=copy(timetostr(encodetime(0,IdleTime*2 div 60,IdleTime*2 mod 60,0)),4,5)+' Min.';
end;

procedure TSetupForm.ShowTimeBarChange(Sender: TObject);
begin
    ShowTime:=ShowTimeBar.Position*1000;
    ShowTimeLabel.caption:=copy(timetostr(encodetime(0,(ShowTime div 1000) div 60,(ShowTime div 1000) mod 60,0)),4,5)+' Min.';
end;

procedure TSetupForm.OverviewTimeBarChange(Sender: TObject);
begin
    OverviewTime:=OverviewTimeBar.Position*1000;
    OverviewTimeLabel.caption:=copy(timetostr(encodetime(0,(OverviewTime div 1000) div 60,(OverviewTime div 1000) mod 60,0)),4,5)+' Min.';
end;

procedure TSetupForm.AbortButtonClick(Sender: TObject);
begin
     application.Terminate;
end;

procedure TSetupForm.OKButtonClick(Sender: TObject);
var
  VFDIni: TIniFile;
  IniName: string;
begin

    LPTPort:=strtoint('$'+PortEdit.Text);

    try
    IniName:=extractfilepath(application.ExeName)+'Vfd.ini';

    VFDIni := TIniFile.Create(IniName);
    with VFDIni do begin
       WriteInteger('EXTRAS', 'ShowTime', ShowTime);
       WriteInteger('EXTRAS', 'OverviewTime', OverviewTime);
       WriteInteger('SETTINGS', 'Port', LPTPort);
       WriteBool('ANIMATIONS', 'PlayOnlyOnIdle', PlayOnlyOnIdleBox.Checked);
       WriteInteger('ANIMATIONS', 'IdleTime', IdleTime);
       WriteInteger('ANIMATIONS', 'IdleLevel', IdleLevel);
    end
    finally
     VFDIni.Free;
    end;

    application.Terminate;
end;

end.
